# plotting.py
# AE3M33UI - Artificial Intellignece course, FEE CTU in Prague
#
# Module containing plotting functions.

from matplotlib.pyplot import *
from sklearn import metrics

def plot_xy_data(x, y, c='b', xlbl='x', ylbl='y', s=20):
    """Create plot of x,y pairs."""
    scatter(x, y, c=c, s=s)
    xlabel(xlbl, fontsize=16)
    ylabel(ylbl, fontsize=16)
    
def plot_xy_classified_data(x, y, c, xlbl='x', ylbl='y', colors=None):
    """Create plot of x,y pairs with classes c indicated by different colors."""
    if colors is None:
        colors = ['b','r','g','y','c','m']
    unique_classes = set(c)
    for k in unique_classes:
        scatter(x[c==k],y[c==k], c=colors[k], s=36)
    xlabel(xlbl, fontsize=16)
    ylabel(ylbl, fontsize=16)

def plot_1D_regr_model(model, c='b'):
    """Display predictions of the model in the x-range of an existing figure."""
    # Get the limits of axes in the current graph
    ax = axis()
    # Build a set of points for which we'd like to display the predictions
    Xtst = np.linspace(ax[0], ax[1], 201)
    Xtst = Xtst[:,np.newaxis]
    # Compute the predictions
    ytst = model.predict(Xtst)
    # Plot them
    p = plot(Xtst, ytst, lw=3, c=c)
    axis(ax)
    return p
    
def plot_2D_class_model(model):
    """Plot the predictions and decision boundary of the model.
    
    Assumes that a plot of data points already exists.
    """
    ax = axis()
    x1 = np.linspace(ax[0], ax[1], num=101)
    x2 = np.linspace(ax[2], ax[3], num=101)
    mx1, mx2 = np.meshgrid(x1, x2)
    sh = mx1.shape
    vx1 = np.reshape(mx1, (-1,1))
    vx2 = np.reshape(mx2, (-1,1))
    vx = np.hstack((vx1,vx2))
    vyhat = model.predict(vx)
    myhat = np.reshape(vyhat, sh)
    contourf(mx1,mx2,myhat,lw=3,cmap=cm.cool, alpha=0.3)

def plot_roc(models, Xtr, ytr, Xtst, ytst):
    """Compute and plots ROC curves of models given as arguments.
    
    The function takes an list of classifiers, it trains each of them on 
    training data, and plots ROC for each classifier as measured on the
    testing data.
    
    Parameters:
    * models : A list of dictionaries describing the models for which ROC shall 
               be plotted. Each dictionary should have the following items:
               'clf' - The classifier itself.
               'descr' - A name or textual description of classifier.
               'color' - The color which shall be used for this model.
    * Xtr, ytr : Training data set used to train the classifiers.
    * Xtst, ytst : Testing data set, ROC is measured using this data.
    
    """
    for model in models:
        # Process the model information
        clf = model['clf']
        descr =  model['descr']
        color = model['color']
        # Fit the classifier to training data
        clf.fit(Xtr, ytr)
        # Get the ROC curve for the classifier
        fpr, tpr, thresholds = metrics.roc_curve(ytst, clf.predict_proba(Xtst)[:,1])
        # Get the AUC
        roc_auc = metrics.auc(fpr, tpr)
    
        # Plot ROC curve
        legstr = '{:s}, AUC = {:.2f}'.format(descr, roc_auc)
        plot(fpr, tpr, color, label=legstr, lw=2)

    # Decorate the graph
    plot([0, 1], [0, 1], 'k--')
    xlim([0.0, 1.0])
    ylim([0.0, 1.0])
    xlabel('False Positive Rate')
    ylabel('True Positive Rate')
    title('Receiver operating characteristic example')
    legend(loc="lower right")
    show()
    
    
def plot_learning_curve(tr_sizes, tr_errors, tst_errors, tr_color='b', tst_color='r'):
    """Plot the learning curve from pre-computed data."""

    fig, ax = subplots()
        
    ax.plot(tr_sizes,tr_errors, lw = 2, c=tr_color, label='training error')
    ax.plot(tr_sizes,tst_errors, lw = 2, c=tst_color, label='cross validation error')
    ax.set_xlabel('training set size')
    ax.set_ylabel('error')
    
    ax.legend(loc = 0)
    ax.set_xlim(0, np.max(tr_sizes))
    ax.set_ylim(0,1)
    ax.set_title('Learning Curve' )    

    